/*
	Name: Lzr Tool v1.0 2019/9/3 13:45
	Copyright:
	Author: YanXi
	Contact: 1197129380 [at] qq.com
	Date: 05-09-19 13:44
	Description: A tool to (un)compresse lzr compressed data used by Gust
*/

#include "msg.h"
#include "lzr.h"
#include "unlzr.h"
#include "common.h"

#include <filesystem>
#include <locale>

//#include <queue>
//#include <vector>

//static bool         quiet     = false;
static bool force_output = false;

int main(int argc, char ** argv)
{
	set_lang();

	std::string input_filename, output_filename;
	int mode = -1, depth_limit = 16;
	if(argc == 1)
	{
		msg (MSG_COPYRIGHT);
		msg (MSG_USAGE);
		return 0;
	}
	else if(argc == 2)
	{
		input_filename  = argv[1];
		if(yx::File::get_extention(input_filename) != std::string("lzr"))
		{
//			fprintf(stderr, "File Extention Error!\n");
			output_filename = input_filename + ".lzr";
			msg(MSG_MODE_COMP);
			mode = M_C;
		}
		else
		{
			msg(MSG_MODE_EXTRACT);
			output_filename = yx::File::get_prefix(input_filename);
			mode = M_UC;
		}
	}
	else
	{
		for(int _I = 1; _I < argc; ++_I)
		{
			if(argv[_I][0] != '-')
			{
				err (ERR_INVALID_PARAM);
				exit(E_INVALID_PARAM);
			}
			switch(argv[_I][1])
			{
				case 'i':
					input_filename = argv[ ++_I ];
					break;
				case 'o':
					output_filename = argv[ ++_I ];
					break;
				case 'r':
					if (1 != sscanf(argv[++_I], "%d", &LZ_REPT_THRESHOLD))
						err(ERR_R_INVALID_PARAM);
					break;
				case 'd':
					if (1 != sscanf(argv[++_I], "%d", &depth_limit))
						err(ERR_D_INVALID_PARAM);
					break;
				case 'e':
					mode = M_UC;
					msg(MSG_MODE_EXTRACT);
					break;
				case 'c':
					mode = M_C;
					msg(MSG_MODE_COMP);
					break;
				case 'f':
					force_output = true;
					break;
				case '?':
					msg(MSG_USAGE);
					break;
				default:
					err(ERR_INVALID_PARAM);
					exit(E_INVALID_PARAM);
			}
		}
		if(output_filename.empty())
		{
			if(mode == M_UC && (input_filename.ends_with(".lzr")))
				output_filename = yx::File::get_prefix(input_filename);
			else output_filename = input_filename + ".lzr";
		}
	}//cmd input proccess section over
	if (mode == -1)
	{
		err(ERR_MODE_NOT_SET);
		exit(E_INVALID_PARAM);
	}

	msg(MSG_FILENAME_ECHO, sys_mbs_to_wcs(input_filename).c_str(), sys_mbs_to_wcs(output_filename).c_str());

	if (!force_output)
	{
		if (std::filesystem::exists(output_filename))
		{
			char ch;
			msg (MSG_OVERWRITE_HINT, sys_mbs_to_wcs(output_filename).c_str());
			while (EOF != scanf("%c", &ch))
			{
				if (ch == '\n' || ch == 'N' || ch == 'n')
				{
					msg(MSG_USER_CANCELED);
					exit(0);
				}
				if (ch == 'Y' || ch == 'y')
				{
					break;
				}
			}
			if (ch != 'Y' && ch != 'y')
			{
				msg(MSG_USER_CANCELED);
				exit(0);
			}
		}
	}

	FILE *ipf = yx::IO::open_convicted(input_filename, "rb"),
	    *opf = yx::IO::open_convicted(output_filename, "wb");

	if(mode == M_UC)   //uncompress
	{
		LZRHD              lzrh;
		unsigned char*     out      = NULL;

		fread(&lzrh, sizeof(lzrh), 1, ipf);

		out  = new unsigned char[lzrh.orig_leng];
		unsigned char* lzrdata  = new unsigned char[lzrh.data_leng];
		unsigned long  flag_len = (lzrh.flag_bits + 7) / 8;
		unsigned char* lzrflag  = new unsigned char[flag_len];

		fread(lzrflag, flag_len, 1, ipf);
		fread(lzrdata, lzrh.data_leng, 1, ipf);

		fclose( ipf );

		if(strcmp((char*)lzrh.symb, "LZR"))
		{
			err(ERR_MAGICHEAD_ERROR);
			exit(E_INVALID_IN);
		}
		debug(L"flag bits:%lu data length:%lu original length:%lu\n\n", lzrh.flag_bits, lzrh.data_leng, lzrh.orig_leng);

		unlzr(&lzrh,
		      lzrflag,
		      lzrdata,
		      out);

		yx::IO::write_convicted(opf, out, lzrh.orig_leng);
		fclose(opf);
	}
	else
	{
		unsigned long long file_len = yx::IO::filelength( ipf );
		unsigned char*     orig_dat = new unsigned char[file_len];
		fread(orig_dat, file_len, 1, ipf);
		fclose(ipf);

		byte_t *rstdata;
		int length = lzr(orig_dat, file_len, &rstdata);
		msg(MSG_COMP_COMP, length);
		yx::IO::write_convicted(opf, rstdata, length);
		fclose(opf);
	}
	msg (MSG_TIME_USED, (long long)clock()/CLOCKS_PER_SEC);
	return 0;
}

