#include "msg.h"
#include "common.h"
#include <clocale>
#include <string>
#include <cassert>

struct message
{
    int id;
    const wchar_t *msg;
};

struct message lang_mes[][MSG_MAX + 1] = {
#include "mes_lang.inc"
};

std::string sys_wcs_to_mbs(const std::wstring &str) noexcept
{
    mbstate_t state{};
    const wchar_t *pstr = str.c_str();
    size_t size = wcsrtombs(NULL, &pstr, 0, &state);
    if (size == (size_t)-1) return "";
    assert(mbsinit(&state));
    char *buf = new char[size + 1];
    wcsrtombs(buf, &pstr, size + 1, &state);
    buf[size] = 0;
    std::string ret{ buf };
    delete[] buf;
    return ret;
}

std::wstring sys_mbs_to_wcs(const std::string &str) noexcept
{
    mbstate_t state{};
    const char *pstr = str.c_str();
    size_t size = mbsrtowcs(NULL, &pstr, 0, &state);
    if (size == (size_t)-1) return L"";
    assert(mbsinit(&state));
    wchar_t *buf = new wchar_t[size + 1];
    mbsrtowcs(buf, &pstr, size + 1, &state);
    buf[size] = 0;
    std::wstring ret{ buf };
    delete[] buf;
    return ret;
}

const wchar_t *get_msg(int id)
{
    struct message *msg = lang_mes[lang_id];
    while (msg->id < id)
    {
        msg++;
    }
    return msg->msg;
}

int lang_id = -1;
struct lang_tbl_itm
{
    const char *lang;
    int id;
} lang_tbl [] =
{
    {"en_US", 0},
    {"zh_CN", 1},
    {"Chinese (Simplified)_China", 1},
    {nullptr, -1}
};

bool prefix_cmp(const char *prefix, const char *target)
{
    const char *ptr = prefix;
    const char *np = target;
    while (*ptr != '\0')
    {
        if (*ptr++ != *np++)
            return false;
    }
    return true;
}

int map_lang_id(const char *name)
{
    struct lang_tbl_itm *cur = lang_tbl;
    while (cur->lang != nullptr)
    {
        if (prefix_cmp(cur->lang, name))
        {
            return cur->id;
        }
        ++cur;
    }
    assert(0);
    return 0;
}

int set_lang()
{
    const char *lang_str = std::setlocale(LC_ALL, "");
    return lang_id = map_lang_id(lang_str);
}

